#!/usr/local/bin/wish

global DIR
set DIR [pwd]


# Set window title
wm title . {Toxic}
# Create and pack a main frame for the GUI
frame .mf
pack .mf

# Create menu buttons
button .mf.tox -relief raised -text Tox -underline 0 -command ttxSelect
label .mf.tox_label -text {Convert TTX file to TOX }
button .mf.detox -relief raised -text Detox -underline 0 -command toxSelect
label .mf.detox_label -text {Convert TOX file to TTX }
label .mf.description1 -text {After converting back to TTX, remove the ".exp" extension from the TTX file. }
label .mf.description2 -text {For usage, refer to the readme file. }

# Pack (grid) menu buttons and labels

grid .mf.tox -column 0 -row 0 -sticky we
grid .mf.tox_label -column 1 -row 0 -sticky w
grid .mf.detox -column 0 -row 2 -sticky we
grid .mf.detox_label -column 1 -row 2 -sticky w
grid .mf.description1 -columnspan 2 -row 3 -sticky we
grid .mf.description2 -column 0 -row 4 -sticky we



##############################
# ttxSelect
# file selection dialog for TTX file

proc ttxSelect {} {

global DIR

set fltyp {
{ "TTX file"	{ *.ttx }	} 
}

set ttxFilename [tk_getOpenFile  -filetypes $fltyp -parent .]

tox $ttxFilename

}


##############################
# toxSelect
# file selection dialog for TOX file

proc toxSelect {} {

global DIR

set fltyp {
{ "TOX file"	{ *.tox }	} 
}

set toxFilename [tk_getOpenFile  -filetypes $fltyp -parent .]

detox $toxFilename

}


##############################
# tox routine

proc tox {ttxFile} {

global srcLang trgLang
global tagno
global tagarray
global segno
global segarray

# initialize tag number

set tagno 1

# initialize target segment number

set segno 1

set langs [tlangident $ttxFile]

set srcLang [lindex $langs 0]

set trgLang [lindex $langs 1]

tfilehandler $ttxFile

}


proc tfilehandler {ttxFile} {

global tagno
global tagarray
global segno
global segarray
global srcLang trgLang
global tempFile

set openTTXComment {<!--leading_suppressed_ttx_body}
set closeTTXComment {trailing_suppressed_ttx_body-->}
set openToxSegComment {<!--leading_suppressed_toxtags}
set closeToxSegComment {trailing_suppressed_toxtags-->}
set patDeclr {<\?xml(.*?)>}


puts "LAUNCHING FILE HANDLER"

set ttxRoot [file rootname $ttxFile]

set toxExt {.tox}

set toxFile $ttxRoot$toxExt


# create and open temporary file

set tempFile {temp.tox}

set tempFh [open $tempFile w]

fconfigure $tempFh -encoding unicode


# open ttx file for line-by-line reading

set ttxFh [open $ttxFile]

fconfigure $ttxFh -encoding unicode

gets $ttxFh 1stLine

if { [regexp $patDeclr $1stLine] == 1} {

puts $tempFh $1stLine 

} else {

puts "no XML declaration found"

exit }

# write leading XML comment

puts $tempFh $openTTXComment


# get each line of ttx file

while { [gets $ttxFh totalLine] >= 0 } {

# substitute xml comments

regsub -all -- {<!--} $totalLine {open_xml_comment} totalLine

regsub -all -- {-->} $totalLine {close_xml_comment} totalLine

regsub -all -- {&lt;!--} $totalLine {open_escaped_xml_comment} totalLine

regsub -all -- {--&gt;} $totalLine {close_escaped_xml_comment} totalLine

# substitute source paragraph tags 
# and problematic characters

set procdLine [tprocline $totalLine]

# write line to output file

puts $tempFh $procdLine

	}

# write closing XML comment for TTX body

puts $tempFh $closeTTXComment

# write opening XML comment for segments

puts $tempFh $openToxSegComment

# insert array of numbers and tags

foreach n [array names tagarray] { 

puts $tempFh <toxtag$n>$tagarray($n)</toxtag$n>

}

puts $tempFh $closeToxSegComment

# insert array of target segments

puts $tempFh {<toxictarget>}

foreach n [lsort -dictionary [array names segarray]] { 

puts $tempFh "<p toxseg$n><p toxseg>$segarray($n)</p toxseg></p toxseg$n>"

	}

puts $tempFh {</toxictarget>}


close $ttxFh

close $tempFh

# open TOX file again and
# revert character substitution

trevertsub $toxFile

file delete temp.tox

}



# proc tlangident
# receives: name of TTX or TOX file
# outputs: source and target language codes

proc tlangident {tFile} {

global srcLang trgLang


puts "LAUNCHING TLANGIDENT"


# open ttx file and read entire content

set tFh [open $tFile]

fconfigure $tFh -encoding unicode

set tSize [file size $tFile]

set tCont [read $tFh $tSize]

# identify source and target language codes

set srcPat {SourceLanguage="(.+?)"}

regexp $srcPat $tCont match srcLang

set trgPat {TargetLanguage="(.+?)"}

regexp $trgPat $tCont match trgLang

set langs [list $srcLang $trgLang]

return $langs


# close ttx file again

close $tFh

}


proc trevertsub {toxFile} {

global tempFile

# re-open temporary file, create and open tox file

set tempFh [open $tempFile r+]

fconfigure $tempFh -encoding unicode

set toxFh [open $toxFile w]

fconfigure $toxFh -encoding unicode


while { [gets $tempFh lineRevisited] >= 0 } {

# revert escaped angle brackets
# revert AND symbol

regsub -all {<andelteesc} $lineRevisited {\&lt;} lineRevisited

regsub -all {andgeeteesc>} $lineRevisited {\&gt;} lineRevisited

regsub -all AANNDD $lineRevisited {\&} lineRevisited

puts $toxFh $lineRevisited

}

close $tempFh

close $toxFh

}



# proc tprocline
# receives: unprocessed total line
# returns: prepared total line

proc tprocline {totalLine} {

global tagno
global segno
global tagarray
global segarray
global srcLang trgLang


# substitute escaped angle brackets
# substitute AND symbol

regsub -all {&lt;} $totalLine {<andelteesc} totalLine

regsub -all {&gt;} $totalLine {andgeeteesc>} totalLine

regsub -all {&} $totalLine AANNDD totalLine


# look for target segments in line

set trgPat1 {(.*?)(<Tuv Lang=")(}
set trgPat2 $trgLang
set trgPat3 {)(">)(.*?)(</Tuv>)(.*?)}

set patTarget $trgPat1$trgPat2$trgPat3

if { [regexp $patTarget $totalLine] == 0 } {

# if non found, return line verbatim

puts "PASSING LINE WITH UNCHANGED TUs"
return $totalLine

} else {

# if target segment found

while { [regexp $patTarget $totalLine] == 1 } {

# parse line for target segment

puts "PARSING TU"

regexp $patTarget $totalLine match sub1 sub2 sub3 sub4 sub5 sub6 sub7

# perform substitutions on target segment

puts "SUBSTITUTIONS ON TU"

set sub5 [ttagplaceholder $sub5]

set oSegPh {<p toxseg}

set cSegPh {/>}

set segarray($segno) $sub5

regsub $patTarget $totalLine $sub1$oSegPh$segno$cSegPh$sub7 totalLine

# sub5 is target segment (within target tags), with unprocessed tags

incr segno

		}

puts "PASSING LINE WITH CHANGED TUs"

return $totalLine


	}

}



# proc ttagplaceholder
# receives: target segment (with full tags)
# outputs: target segment (with collapsed tags)

proc ttagplaceholder {sub5} {

global tagno
global tagarray
global srcLang trgLang

set tagPhPat1 {(<[^<>]+>)+}

set tagPhPat2 {xxtoxtag}

set tagPhPat3 yy

set tagPhPat4 {xxtoxtag([0-9]+)yy}

set tagPhPat5 {<toxtag\1/>}

while { [regexp $tagPhPat1 $sub5] == 1 } {

regexp $tagPhPat1 $sub5 tagseq

set tagarray($tagno) $tagseq

regsub $tagPhPat1 $sub5 $tagPhPat2$tagno$tagPhPat3 sub5

incr tagno

	}

regsub -all $tagPhPat4 $sub5 $tagPhPat5 sub5

return $sub5

}




##############################
# detox routine


proc detox {toxFile} {

# create segment and tag arrays

dtgetarrays $toxFile

set trgLang [dtlangident $toxFile]

dtpreproc $toxFile

# pass filename to dtfilehandler

dtfilehandler $toxFile

dtrevertchars

file delete temp1.tox

file delete temp2.tox

}



# proc dtpreproc
# recreates tox file without 
# XML comments and arrays

proc dtpreproc {toxFile} {

# open tox file for line-by-line reading

set striP1 {(ttx_body)|}
set striP2 {(ttx_body)|}
set striP3 {(leading_suppressed_toxtags)|}
set striP4 {(trailing_suppressed_toxtags)|}
set striP5 {(toxictarget>)|}
set striP6 {(</toxtag)|}
set striP7 {(</p toxseg)}

# look for lines required only by the procedure

set striPat $striP1$striP2$striP3$striP4$striP5$striP6$striP7


set toxFh [open $toxFile]

fconfigure $toxFh -encoding unicode

# create and open temporary file 1

set temp1File {temp1.tox}

set temp1Fh [open $temp1File w]

fconfigure $temp1Fh -encoding unicode


while { [eof $toxFh] != 1 } { 

gets $toxFh newLine

puts {PREPROCEDTOX READING LINE BY LINE}

if { [regexp $striPat $newLine] == 1 } {

puts {DELETING THIS:}

puts $newLine

} else {

set procLine [dtharmlesschars $newLine]

puts $temp1Fh $procLine

puts {KEEPING THIS:}

puts $procLine

		}

	}

close $toxFh

close $temp1Fh

}




# proc dtlangident
# receives: name of TTX or TOX file
# outputs: source and target language codes

proc dtlangident {tFile} {

global srcLang trgLang


puts "LAUNCHING DTLANGIDENT"


# open ttx/tox file and read entire content

set tFh [open $tFile]

fconfigure $tFh -encoding unicode

set tSize [file size $tFile]

set tCont [read $tFh $tSize]

# identify source and target language codes

set srcPat {SourceLanguage="(.+?)"}

regexp $srcPat $tCont match srcLang

set trgPat {TargetLanguage="(.+?)"}

regexp $trgPat $tCont match trgLang

set langs [list $srcLang $trgLang]

# close ttx file again

close $tFh

puts "TARGET LANGUAGE IS $trgLang"

return $trgLang

}



proc dtfilehandler {toxFile} {

global temp2File
global ttxFile
global trgLang
global segarray
global tagarray

set LINENUMBER 0
set temp1File temp1.tox

puts "LAUNCHING FILE HANDLER"

set patOpenTTXComment {leading_suppressed_ttx_body}
set patCloseTTXComment {trailing_suppressed_ttx_body}

set segPhPat {<p toxseg([0-9]+)/>}
set patDeclr {<\?xml(.*?)>}

set oTrgTag1 {<Tuv Lang="}
set oTrgTag2 {">}
set oTrgTag $oTrgTag1$trgLang$oTrgTag2
set cTrgTag {</Tuv>}

set toxRoot [file rootname $toxFile]

set ttxExt {.ttx}

set expSuffix {.exp}

set ttxFile $toxRoot$ttxExt$expSuffix


# create and open temporary file 2

set temp2File {temp2.tox}

set temp2Fh [open $temp2File w]

fconfigure $temp2Fh -encoding unicode


# open temp1 file for line-by-line reading

set temp1Fh [open temp1.tox]

fconfigure $temp1Fh -encoding unicode

# read XML declaration

gets $temp1Fh 1stLine

if { [regexp $patDeclr $1stLine] == 1 } {

puts $temp2Fh $1stLine 

} else {

puts "no XML declaration found"

exit 
	}


# read file until eof

while { [gets $temp1Fh totalLine] != -1 } {

# look for segment placeholder in line

while { [regexp $segPhPat $totalLine] == 1 } {

puts "LOOKING FOR SEGMENT PLACEHOLDER"

# sub1 is segment number

regexp $segPhPat $totalLine match sub1

# using segment number, get target segment from array

set trgSeg $segarray($sub1)

puts {TARGET SEGMENT FOR THIS SEGMENT NUMBER IS:}
puts $trgSeg

# trgSeg is target segment with inline placeholders

# pass segment to tag processor
# get segment back with substituted tags

puts {ABOUT TO SUBSTITUTE INLINE TAGS}

# launch inline tag substitution routine

set insSeg [dtproctag $trgSeg]

# insSeg is target segment with verbose inline tags

puts {TARGET SEGMENT WITH SUBSTITUTED TAGS:}
puts $insSeg

# substitute escaped angle brackets
# substitute AND symbol

# insert target segment into line

regsub $segPhPat $totalLine $oTrgTag$insSeg$cTrgTag totalLine


puts {LINE WITH TARGET SEGMENt AND SUBSTITUTED TAGS}

# end of segment reading loop

		}

# write line to temporary file

puts {WRITING LINE TO TEMPORARY FILE}

puts $temp2Fh $totalLine

incr LINENUMBER

# end of line reading loop

	}

close $temp1Fh

close $temp2Fh

# end of procedure

}



# proc dtgetarrays
# receives: name of TOX file
# creates: segarray and tagarray

proc dtgetarrays {toxFile} {

global tagarray
global segarray
global trgLang

puts "LAUNCHING DTGETARRAYS"

# set toxic tag and toxic segment patterns

set tagPat {(<toxtag)([0-9]+)(>)(.*?)(</toxtag)([0-9]+)>}
set segPat {(<p toxseg)([0-9]+)(><p toxseg>)(.*?)(</p toxseg>)(</p toxseg)([0-9]+)(>)}
set endPat {</toxictarget>}

# open tox file for line-by-line reading

set toxFh [open $toxFile]

fconfigure $toxFh -encoding unicode

# read tox file line by line

while { [gets $toxFh totalLine] >= 0 } {

# look for toxic tags

if { [regexp $tagPat $totalLine] == 1 } {

regexp $tagPat $totalLine match sub1 sub2 sub3 sub4

# sub2 is the tag number
# sub4 is the tag content

set tagNo $sub2

# substitute characters with harmless characters

set tagCont [dtharmlesschars $sub4]

# add to array

set tagarray($tagNo) $tagCont

} elseif { [regexp $segPat $totalLine] == 1 } {

regexp $segPat $totalLine match sub1 sub2 sub3 sub4

# sub2 is the target segment number
# sub4 is the target segment content

set segNo $sub2

set segCont $sub4

# substitute characters with harmless characters

set segCont [dtharmlesschars $sub4]

# add to array

set segarray($segNo) $segCont

		}

	}

close $toxFh

}


# proc dtproctag
# receives: target segment (with collapsed tags)
# outputs: target segment (with full tags)

proc dtproctag {collapsedSeg} {

global tagarray

set tagPhPat {<toxtag([0-9]+)/>}

while { [regexp $tagPhPat $collapsedSeg] == 1 } {

regexp $tagPhPat $collapsedSeg match tagno

puts "PROCESSING TAG NUMBER $tagno"

# tagno is tag number

set inlineTagSeq $tagarray($tagno)

regsub $tagPhPat $collapsedSeg $inlineTagSeq collapsedSeg

	}

set expandedSeg $collapsedSeg

return $expandedSeg

}



# proc dtrevertchars
# revert substituted characters
# as revertsub, but for tox -> ttx

proc dtrevertchars {} {

global temp2File
global ttxFile

# re-open temporary file, create and open tox file

set temp2Fh [open $temp2File r+]

fconfigure $temp2Fh -encoding unicode

set ttxFh [open $ttxFile w]

fconfigure $ttxFh -encoding unicode


while { [gets $temp2Fh lineRevisited] >= 0 } {

# revert escaped angle brackets
# revert AND symbol

regsub -all {<andelteesc} $lineRevisited {\&lt;} lineRevisited

regsub -all {andgeeteesc>} $lineRevisited {\&gt;} lineRevisited

regsub -all AANNDD $lineRevisited {\&} lineRevisited

# revert substituted xml comments

regsub -all {open_xml_comment} $lineRevisited {<!--} lineRevisited

regsub -all {close_xml_comment} $lineRevisited {-->} lineRevisited

regsub -all -- {open_escaped_xml_comment} $lineRevisited {&lt;!--} lineRevisited

regsub -all -- {close_escaped_xml_comment} $lineRevisited {--&gt;} lineRevisited


puts $ttxFh $lineRevisited

	}

close $temp2Fh

close $ttxFh

}


proc dtharmlesschars {dangerString} {

# substitute characters temporarily

# substitute escaped angle brackets
# substitute AND symbol

regsub -all {&lt;} $dangerString {<andelteesc} harmlessString

regsub -all {&gt;} $dangerString {andgeeteesc>} harmlessString

regsub -all {&} $dangerString AANNDD harmlessString

return $harmlessString

}

